// Esempio di Passaagio a livello
// tradotto in italiano e modifiche di Enzo Fortuna
// progetto originale di Geoff Bunza, Version 1.3 2014
//
// Usa la libreria "SERVO" (già inclusa di default in Arduino-IDE) per la gestione dei servo
//
// Occupa circa 3,6 K di memoria su 30 K disponibili.
//
// 2 Servo sono connessi ai pin 3 e 4 Digitali di un Arduino NANO ( 5Volt, 16 Mhz )
// 2 sensori IR sono connessi ai pin 5 e 6, 
// 4 LED lampeggianti ( 2 coppie) connessi ai pin 10 ed 11, e 12 e 13 (due led alternati ai due lati del PL
// Usare 2 servo tipo 9G
// per evitare spikes sulla alimetazione conviene aggiungere un paio di condensator da 100nF ceramici
// ed un elettrolitico da almeno 47uF vicino ai piedini di alimentazione del Nano.
//
// Funzionamento:
// Quando viene rilevato un treno da uno dei due sensori IR 
// I lampeggianti vengono azionati immediatamente e continuuano per 3 secondi,
// Quindi si azionano verso il basso le sbarre lentamente.
// I lampeggianti continuano con le sbarre giù fino a che il secondo sensore è attivo (treno che passa)
// e dopo che il secondo sensore non rileva nula per almeno altri 1.2 secondi.
// Quindi le sbarre vengono alzate e le luci vengono spente.
// Il treno può arrivare da entrambe le direzioni, e deve passare completamente attraverso il PL
// è possibile aggiungere dove indicato una uscita che comanda un generatore audio per il suono di una campana di avviso

#include <Servo.h>

Servo gate1servo;  // crea l'oggetto "servo" per le sbarre 1
//SI ASSUME che le sbarre siano calate in posizione 30 e alzate completamente in posizione 120
Servo gate2servo;  // crea l'oggetto "servo" per le sbarre 2

int sensor1 = 5;   // Sensore IR pin == basso (LOW) quando rilevato un treno
int sensor2 = 6;   // Sensore IR pin == basso (LOW) quando rilevato un treno
int led1 = 10;     // Led 1 - LED Lampeggiante lato A
int led2 = 11;     // Led 2 - LED Lampeggiante lato A
int led3 = 12;     // Led 3 - LED Lampeggiante lato B
int led4 = 13;     // Led 4 - LED Lampeggiante lato B
int gateposition = 120;    // variabile che ricorda la posizione dei servo (Alzate allo start)
int entering_sensor = 5;       //Indica quale sensore è attivo per primo
int leaving_sensor = 6;        // Indica quale sensore deve controllare che il treno sia finito di passare
int gates_started = 0;         // Indica se il passaggio a livello è attivo
int flash_state = 0;
long flash_time = 0;
long  flash_interval = 900;    // tempo in millisecondi tra lampeggio dei LED alternati
int sensor_count = 0;

void setup()
{
  gate1servo.attach(3);  // attacca il servo sul pin 3 all'oggetto "servo"
  gate2servo.attach(4);  // attacca il servo sul pin 4 all'oggetto "servo"
  gate1servo.write(gateposition);  //Inizia assumendo che non ci siano treni
  gate2servo.write(gateposition);  //Inizia assumendo che non ci siano treni
  pinMode(sensor1, INPUT);  
  pinMode(sensor2, INPUT);
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);  
  pinMode(led3, OUTPUT);
  pinMode(led4, OUTPUT);
  digitalWrite(led1, LOW);  // Inizia con tutti i LED spenti
  digitalWrite(led2, LOW);
  digitalWrite(led3, LOW);
  digitalWrite(led4, LOW); 
  flash_time = millis();
}

void loop()
{
  if ((digitalRead (sensor1)==LOW)&& (gates_started==0)) {
    gates_started = 1;
    leaving_sensor = sensor2;
    starting_sequence();
  }
  if ((digitalRead (sensor2)==LOW)&& (gates_started==0)) {
    gates_started = 1;
    leaving_sensor = sensor1;
    starting_sequence();
  }
  if (gates_started) flash_leds();   //continua il lampeggio fino a "sbarre tutte giù"
  if ((digitalRead(leaving_sensor)==LOW)&&(gates_started==1)) 
  {
    //Il treno inizia a lasciare il passaggio a livello
    //fino a che il secondo sensore è attivo il treno sta ancora passando
    while (gates_started==1)  {   //adesso controlla che il treno sia veramente terminato
      sensor_count = 0;
      for (int i=1; i<40; i++)  {
        if (digitalRead(leaving_sensor)==LOW) sensor_count++;
        delay (30);
        flash_leds();
      }
      if (sensor_count==0) gates_started=0;
      flash_leds();
    }
    // Si arriva qui solo se il treno è veramente finito di passare il PL
    ending_sequence();
  }
}

void starting_sequence()  {
  long wait_time;
  flash_time = millis();
  wait_time = millis()+3000;
  // qui si può attivare l'uscita per la campanella
  while (wait_time > millis())  flash_leds();  //Lampeggia prima di abbassare le sbarre
  for(gateposition = 120; gateposition>30; gateposition-=1)  // che poi vanno da 120 gradi a 30 gradi
  {                               
    gate1servo.write(gateposition);  // dice al servo di muoversi fino alla posizione indicata dalla variabile 'gateposition'
    gate2servo.write(gateposition);  // idem per l'altro servo
    flash_leds();                    // e continua il lampeggio LED
    delay(40);                       // aspetta 40ms per far muovere lentamente i servo
  }
}

void ending_sequence()  {
  for(gateposition = 30; gateposition<120; gateposition++)   // le sbarre vanno da 30 gradi a 120 gradi
  {                               
    gate1servo.write(gateposition);  // dice al servo di muoversi fino alla posizione indicata dalla variabile 'gateposition'
    gate2servo.write(gateposition);  // idem per l'altro servo
    flash_leds();                    // e continua il lampeggio LED
    delay(40);                       // spetta 40ms per far muovere lentamente i servo
  }
  digitalWrite(led1, LOW);  // Lampeggianti tutti Spenti!
  digitalWrite(led3, LOW);
  digitalWrite(led2, LOW);
  digitalWrite(led4, LOW);
  // qui si può spegnere l'uscita della campanella!
}
void flash_leds()  {
  if (flash_time > millis()) return;
  flash_state = ~flash_state;
  digitalWrite(led1, flash_state);  // Alterna i lampeggianti
  digitalWrite(led3, flash_state);
  digitalWrite(led2, ~flash_state);
  digitalWrite(led4, ~flash_state);
  flash_time = millis()+flash_interval;
}


